package com.glxp.udidl.admin.config;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.dao.sys.SysUserMapper;
import com.glxp.udidl.admin.exception.JsonException;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

@Aspect
@Component
public class AuthAspect {
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private GlobalConfig globalConfig;
    @Pointcut("@annotation(com.glxp.udidl.admin.annotation.AuthRuleAnnotation)")
    private void authMethod() {
    }

    @Before(value = "authMethod()")
    public void before(JoinPoint joinPoint) {
        if(globalConfig.isOpenAuth() == false)
            return;
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

        if (attributes == null) {
            throw new JsonException(-1, "attributes = null");
        }
        HttpServletRequest request = attributes.getRequest();

        String key = request.getHeader("key");
        if (key == null || key == "")
            throw new JsonException(401, "key不能为空");
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        AuthRuleAnnotation action = method.getAnnotation(AuthRuleAnnotation.class);
        String perms = action.value();
        Integer res = sysUserMapper.hasAuthPerms(key,perms);
        if(res == 0)
            throw new JsonException(402, "无权限！");
    }
}
