package com.glxp.udidl.admin.controller.device;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udid.Contactlist;
import com.glxp.udidl.admin.entity.udid.Device;
import com.glxp.udidl.admin.req.DlConnactRequest;
import com.glxp.udidl.admin.res.DLConnactResponse;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.DlConnactService;
import com.glxp.udidl.common.enums.ResultEnum;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 医疗器械电话联系人
 */
@RestController
public class ContactController {
    @Resource
    private DlConnactService dlConnactService;
    @Resource
    private DeviceService deviceService;

    @AuthRuleAnnotation("udidl_contact_all")
    @GetMapping("udidl/device/dlconnact")
    public BaseResponse dlconnacts(@Valid DlConnactRequest dlConnactRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Contactlist> contactlistList = dlConnactService.dlConnacts(dlConnactRequest);
        List<DLConnactResponse> dlConnactResponses = new ArrayList<>();
        if (contactlistList != null && contactlistList.size() > 0) {
            for (Contactlist contactlist : contactlistList) {
                Device device = deviceService.searchByDeviceRecordKey(contactlist.getDevicerecordkey());
                DLConnactResponse dlConnactResponse = new DLConnactResponse();
                dlConnactResponse.setDeviceRecordKey(contactlist.getDevicerecordkey());
                dlConnactResponse.setQylxrcz(contactlist.getQylxrcz());
                dlConnactResponse.setQylxrdh(contactlist.getQylxrdh());
                dlConnactResponse.setQylxryx(contactlist.getQylxryx());
                dlConnactResponse.setCpmctymc(device.getCpmctymc());
                dlConnactResponse.setYlqxzcrbarmc(device.getYlqxzcrbarmc());
                dlConnactResponses.add(dlConnactResponse);
            }
        }
        return ResultVOUtils.success(dlConnactResponses);
    }

}
