package com.glxp.udidl.admin.controller.device;

import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udi.UdiCompanyEntity;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.req.UdiCompanyRequest;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.common.enums.ResultEnum;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;


//UDI查询APP API
@RestController
public class UdiCompanyController {

    @Resource
    UdiCompanyService udiCompanyService;
    @Resource
    ProductInfoService productInfoService;

    @AuthRuleAnnotation("udidl_udiCompany_all")
    @GetMapping("udi/company/filter")
    public BaseResponse filterCompany(@Valid UdiCompanyRequest udiCompanyRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiCompanyEntity> udiCompanyEntities = udiCompanyService.filterUdiCompany(udiCompanyRequest);
        return ResultVOUtils.success(udiCompanyEntities);
    }
    @AuthRuleAnnotation("udidl_udiCompany_all")
    @GetMapping("udi/company/filtertByTD")
    public BaseResponse filtertByTD(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ProductInfoEntity> productInfoEntities = productInfoService.filterUdiByTyshxydm(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntities);
    }

    //udi查询app接口，获取企业最新DI
    @AuthRuleAnnotation("udidl_udiCompany_all")
    @GetMapping("udi/company/filtertUdi")
    public BaseResponse filtertUdi(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<ProductInfoEntity> productInfoEntities;
        productInfoFilterRequest.setDiType("1");
        if (productInfoFilterRequest.isShowHis()) {
            productInfoEntities = productInfoService.filterUdiByTyshxydm(productInfoFilterRequest);
        } else {
            productInfoFilterRequest.setDiType("1");
            productInfoFilterRequest.setIsNewest(1);
            productInfoEntities = productInfoService.filterUdiByNewest(productInfoFilterRequest);
        }

        PageInfo<ProductInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(productInfoEntities);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(productInfoEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }
}
