package com.glxp.udidl.admin.controller.device;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.service.DataSync.DeviceSyncService;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

@RestController
public class UdidlDeviceController {
    private static final Logger logger = LoggerFactory.getLogger(UdidlDeviceController.class);
    @Autowired
    DeviceSyncService deviceSyncService;

    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @GetMapping("udidl/device/dlByDays")
    public BaseResponse dlByDays(@DateTimeFormat(pattern = "yyyy-MM-dd") Date startDate, @DateTimeFormat(pattern = "yyyy-MM-dd") Date endDate) {
        logger.info("---按日期段下载");
        deviceSyncService.downloadUdi(startDate, endDate);
        return ResultVOUtils.success("后台已开始下载！");
    }

    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @GetMapping("udidl/device/dlByDay")
    public BaseResponse dlByDay(String day) {
        logger.info(day + "---按天开启下载");
        deviceSyncService.downloadUdi(day, "manual");
        //asyncDownloadTask.downloadByDay(day);
        return ResultVOUtils.success("后台已开始下载！");
    }

    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @PostMapping("udidl/device/dlByDi")
    public BaseResponse downloadByDi(String deviceId) {
        return deviceSyncService.downloadByDi(deviceId);
    }

    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @GetMapping("udidl/device/serchDlByDi")
    public BaseResponse serchDlByDi(String deviceId) {
        return deviceSyncService.searchDlByDi(deviceId);
    }


    /**
     * 获取产品标识详情
     *
     * @param deviceId
     * @return
     */
    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @PostMapping("udidl/device/downloadSingle")
    public BaseResponse downloadSingle(String deviceId) {
        return deviceSyncService.downloadSingle(deviceId);
    }

    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @PostMapping("udidl/device/downloadHistory")
    public BaseResponse downloadHistory(String key) {
        return deviceSyncService.downloadHistory(key);
    }
}
