package com.glxp.udidl.admin.service.DataSync;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.glxp.udidl.admin.entity.info.CompanyEntity;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.Device;
import com.glxp.udidl.admin.entity.udid.JobLog;
import com.glxp.udidl.admin.entity.udid.TokenEntity;
import com.glxp.udidl.admin.req.udid.DeviceRequest;
import com.glxp.udidl.admin.req.udid.DownloadDiRequest;
import com.glxp.udidl.admin.req.udid.DownloadHistoryRequest;
import com.glxp.udidl.admin.req.udid.TokenRequest;
import com.glxp.udidl.admin.res.udid.DataSetHistoryResult;
import com.glxp.udidl.admin.res.udid.DataSetResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleResult;
import com.glxp.udidl.admin.res.udid.DataSetTotalResult;
import com.glxp.udidl.admin.service.info.CompanyService;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.HttpClient;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 设备信息下载
 */
@Service
public class DeviceDownloadService {
    final Logger logger = LoggerFactory.getLogger(DeviceDownloadService.class);
    private String token = "";
    int reCount = 0;
    CompanyEntity companyEntity;
    String url = "sharing/get";
    @Autowired
    private CompanyService companyService;
    @Autowired
    private UdiCompanyService udiCompanyService;
    @Autowired
    JobLogService jobLogService;                //日志
    @Autowired
    DeviceSaveService deviceSaveService;
    @Autowired
    private DeviceService deviceService;
    @Autowired
    private ProductInfoService productInfoService;

    /*public DeviceDownloadService() {
        companyEntity = companyService.findCompany();
        if (companyEntity != null && companyEntity.getClasses() != null) {
            if (companyEntity.getClasses().equals("1")) {
                url = "company/identifier/get";
            } else {
                url = "sharing/get";
            }
        }
    }*/
    private void SetTokenEmpty() {
        token = "";
    }

    /**
     * 获取当前token
     *
     * @return
     */
    public String getToken() {
        if (token.isEmpty()) {
            companyEntity = companyService.findCompany();
            TokenRequest tokenRequest = new TokenRequest();
            tokenRequest.setAppId(companyEntity.getAppId());
            tokenRequest.setAppSecret(companyEntity.getAppSecret());
            tokenRequest.setTyshxydm(companyEntity.getTyshxydm());
            String response = HttpClient.post("token/get", tokenRequest);
            TokenEntity tokenEntity = (TokenEntity) JSONObject.parseObject(response, TokenEntity.class);
            token = tokenEntity.getAccessToken();
        }
        return token;
    }

    /**
     * 获取产品标识详情数据
     *
     * @param deviceId
     * @return
     */
    public DataSetSingleResult downloadByDi(String deviceId) {
        DataSetSingleResult result = new DataSetSingleResult();
        DownloadDiRequest downloadDiRequest = new DownloadDiRequest();
        downloadDiRequest.setAccessToken(getToken());
        downloadDiRequest.setPrimaryDeviceId(deviceId);
        String response = "";
        try {
            response = HttpClient.post("sharing/single", downloadDiRequest);
        } catch (Exception e) {
            result.setReturnCode(-1);
            result.setReturnMsg(e.getMessage());
            return result;//发生错误 退出
        }
        return JSONObject.parseObject(response, DataSetSingleResult.class);
    }

    /**
     * 获取单条数据
     *
     * @param rangeValue
     * @param requestType
     * @param page
     * @return
     */
    private DataSetResult getData(String rangeValue, int requestType, int page) {
        DataSetResult result = new DataSetResult();
        DeviceRequest deviceRequest = new DeviceRequest();
        deviceRequest.setAccessToken(getToken());
        deviceRequest.setRequestType(requestType + "");
        deviceRequest.setRangeValue(rangeValue);
        deviceRequest.setCurrentPageNumber(page + "");
        String response = "";
        try {
            Thread.sleep(3 * 1000);
            response = HttpClient.post(url, deviceRequest);
        } catch (InterruptedException e) {
            result.setReturnCode(-1);
            result.setReturnMsg(e.getMessage());
            return result;//发生错误 退出
        }
        return JSONObject.parseObject(response, DataSetResult.class);
    }

    /**
     * 获取数据
     *
     * @param rangeValue  "当按照天请求数据时，必填，输入参数按照 yyyy-MM-dd，当按照月请求数据时，必填，输入参数按照 yyyy-MM
     * @param requestType 请求范围，1 按天请求，2 按月请求
     * @return
     */

    public DataSetTotalResult getData(String rangeValue, int requestType) {
        DataSetTotalResult results = new DataSetTotalResult();
        int page = 1;
        //int totalPage = 1;
        int totalRecordCount = 0;
        boolean hasData = true;//是否还有数据，如无退出
        List<DataSetResult.DataSet> dataSets = new ArrayList<DataSetResult.DataSet>();
        while (hasData == true) {
            DataSetResult dataSetResult = getData(rangeValue, requestType, page);
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {
                token = "";
                reCount++;
                dataSetResult = getData(rangeValue, requestType, page);
            }
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + JSON.toJSONString(dataSetResult) + ";";
                results.setMsg(msg);
                return results;//发生错误 退出
            } else {
                if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    dataSets.add(dataSetResult.getDataSet());
                    totalRecordCount = dataSetResult.getTotalRecordCount();
                } else { //获取不到数据，退出循环
                    hasData = false;
                    break;
                }
            }
            page++;
            //totalPage = dataSetResult.getTotalPageCount();
            totalRecordCount = dataSetResult.getTotalRecordCount();
        }
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        reCount = 0;
        return results;

      /*  while (page <= totalPage) {
            DataSetResult dataSetResult = getData(rangeValue, requestType, page);
            //发出错误，重新获取1次Token再获取1次，
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {
                token = "";
                reCount++;
                dataSetResult = getData(rangeValue, requestType, page);
            }
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + JSON.toJSONString(dataSetResult) + ";";
                results.setMsg(msg);
                return results;//发生错误 退出
            } else if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0)
                dataSets.add(dataSetResult.getDataSet());
            page++;
            totalPage = dataSetResult.getTotalPageCount();
            totalRecordCount = dataSetResult.getTotalRecordCount();
        }
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        reCount = 0;
        return results;*/
    }

    /**
     * 下载一天数据
     *
     * @param day
     * @return
     */
    public DataSetTotalResult getData(String day, String downloadType) {
        DataSetTotalResult result;//= new DataSetTotalResult();
        String rangeValue = "";
        if (day.isEmpty())
            rangeValue = DateUtil.getYesterday();
        else
            rangeValue = day;
        result = getData(rangeValue, 1);
        if (result.getCode() != 1) {
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(downloadType);
            jobLog.setDownloadDate(DateUtil.parseDate(rangeValue));
            jobLog.setType("error");
            jobLog.setMsg(result.getMsg());
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return result;
        }

        List<String> keyList = new ArrayList<String>();
        for (DataSetResult.DataSet ds : result.getData()) {
            //List<DataSetResult.DeviceInfo> list = ds.getDeviceInfo().stream().filter(d -> d.getVersionNumber() > 1).collect(Collectors.toList());
            List<DataSetResult.DeviceInfo> list = ds.getDeviceInfo();//.stream().collect(Collectors.toList());//改为全部下载历史记录
            if (list != null && list.size() > 0) {
                for (DataSetResult.DeviceInfo deviceInfo : list)
                    keyList.add(deviceInfo.getDeviceRecordKey());
            }
        }
        if (keyList.size() > 0) {
            DataSetTotalResult res = getHistoryData2(keyList);
            //System.out.println("获取记录："+result.getTotalRecordCount()+","+ result.realRecordCount());
            if (res.getCode() == 1) {
                if (res.getData() != null && res.getData().size() > 0) {
                    int count = result.getTotalRecordCount();
                    count = count + res.getTotalRecordCount();
                    result.setTotalRecordCount(count);
                    List<DataSetResult.DataSet> data = result.getData();
                    data.addAll(res.getData());
                    result.setData(data);
                    System.out.println("历史记录：" + res.getTotalRecordCount() + "," + res.realRecordCount());
                    System.out.println("总记录：" + result.getTotalRecordCount() + "," + result.realRecordCount());
                }
            } else {
                JobLog jobLog = new JobLog();
                jobLog.setDownloadType(downloadType);
                jobLog.setDownloadDate(DateUtil.parseDate(rangeValue));
                jobLog.setType("error");
                jobLog.setMsg("下载历史记录出错：" + res.getMsg());
                jobLog.setCreateTime(new Date());
                jobLogService.insert(jobLog);
            }

        }
        return result;
    }

    public List<String> getKeyList(DataSetResult.DataSet ds) {
        return ds.getDeviceInfo().stream().map(item -> item.getDeviceRecordKey()).collect(Collectors.toList());
    }

    public DataSetTotalResult getHistoryData2(List<String> deviceRecordKeys) {
        DataSetTotalResult result = new DataSetTotalResult();
        int limit = 10;
        int totalPage = (deviceRecordKeys.size() - 1) / limit + 1;
        int page = 1;
        while (page <= totalPage) {
            List<String> list = deviceRecordKeys.stream().skip((page - 1) * limit).limit(limit).collect(Collectors.toList());
            DataSetTotalResult res = getHistoryData(list);
            if (res.getCode() != 1)
                return res;
            if (res.getData() != null && res.getData().size() > 0) {
                if (res.getTotalRecordCount() != res.realRecordCount())
                    System.out.println("行数不一致：" + res.getTotalRecordCount() + "," + res.realRecordCount());
                result.setTotalRecordCount(result.getTotalRecordCount() + res.getTotalRecordCount());
                List<DataSetResult.DataSet> dataList = result.getData();
                dataList.addAll(res.getData());
                result.setData(dataList);
            }
            page++;
        }
        return result;
    }

    /**
     * 获取历史数据
     *
     * @param deviceRecordKeys
     * @return
     */
    public DataSetTotalResult getHistoryData(List<String> deviceRecordKeys) {
        DataSetTotalResult results = new DataSetTotalResult();
        String historyUrl = "sharing/history/get";
        int page = 1;
        //int totalPage = 1;
        int totalRecordCount = 0;
        DownloadHistoryRequest deviceHistoryRequest = new DownloadHistoryRequest();
        deviceHistoryRequest.setAccessToken(getToken());
        List<DownloadHistoryRequest.DataSetBean> dataSetBeans = new ArrayList<>();
        deviceRecordKeys.forEach(item -> {
            DownloadHistoryRequest.DataSetBean keys = new DownloadHistoryRequest.DataSetBean();
            keys.setDeviceRecordKey(item);
            dataSetBeans.add(keys);
        });
        deviceHistoryRequest.setDataSet(dataSetBeans);
        List<DataSetResult.DataSet> dataSets = new ArrayList<DataSetResult.DataSet>();
        boolean hasData = true;
        while (hasData == true) {
            deviceHistoryRequest.setCurrentPageNumber(page + "");
            String response = "";
            try {
                Thread.sleep(3 * 1000);
                response = HttpClient.post(historyUrl, deviceHistoryRequest);
            } catch (InterruptedException e) {
                e.printStackTrace();
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + e.getMessage() + ";";
                results.setMsg(msg);
                return results;//发生错误 退出
            }
            DataSetHistoryResult dataSetResult = JSONObject.parseObject(response, DataSetHistoryResult.class);
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {//历史记录失败后重新下载1次
                token = "";
                reCount++;
                continue;
            }
            page++;
            //totalPage = dataSetResult.getTotalPageCount();
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + dataSetResult.getReturnMsg();
                results.setMsg(msg);
                return results;//发生错误 退出
            } else {
                /*if (dataSetResult.getDataList().getDeviceInfo() != null && dataSetResult.getDataList().getDeviceInfo().size() > 0){
                    dataSets.add(dataSetResult.getDataList());
                    totalRecordCount = totalRecordCount+dataSetResult.getTotalRecordCount();
                }else*/
                if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    dataSets.add(dataSetResult.getDataSet());
                    totalRecordCount = totalRecordCount + dataSetResult.getTotalRecordCount();
                } else {
                    hasData = false;
                    break;
                }
            }


        }
        reCount = 0;
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        return results;

       /* while (page <= totalPage) {
            deviceHistoryRequest.setCurrentPageNumber(page + "");
            String response = "";
            try {
                Thread.sleep(3 * 1000);
                response = HttpClient.post(historyUrl, deviceHistoryRequest);
            } catch (InterruptedException e) {
                e.printStackTrace();
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + e.getMessage() + ";";
                results.setMsg(msg);
                return results;//发生错误 退出
            }
            //String response = HttpClient.post(url, deviceHistoryRequest);
            DataSetHistoryResult dataSetResult = JSONObject.parseObject(response, DataSetHistoryResult.class);
            *//*if (dataSetResult.getReturnCode() != 1 && reCount == 0) {//历史记录失败后重新下载1次
                token = "";
                reCount++;
                continue;
            }*//*
            page++;
            totalPage = dataSetResult.getTotalPageCount();
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + dataSetResult.getReturnMsg();
                results.setMsg(msg);
                return results;//发生错误 退出
            } else if (dataSetResult.getDataList().getDeviceInfo() != null && dataSetResult.getDataList().getDeviceInfo().size() > 0){
                dataSets.add(dataSetResult.getDataList());
                totalRecordCount = totalRecordCount+dataSetResult.getTotalRecordCount();
            }


        }
        reCount = 0;
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        return results;*/
    }

    public DataSetTotalResult getDataAll(int days) {
        DataSetTotalResult result = new DataSetTotalResult();
        //result.setData(new ArrayList<>());
        for (int i = 1; i <= days; i++) {
            String day = DateUtil.getLastDay(-i);
            DataSetTotalResult res = getData(day, 1);
            if (res.getCode() != 1) {
                result.setCode(res.getCode());
                result.setMsg(res.getMsg());
                return result;
            }
            int count = result.getTotalRecordCount();
            count = count + res.getTotalRecordCount();
            result.setTotalRecordCount(count);
            List<DataSetResult.DataSet> data = result.getData();
            if (data == null)
                data = res.getData();
            else
                data.addAll(res.getData());
            logger.info("data数量：" + data.size());
            result.setData(data);
        }

        List<String> keyList = new ArrayList<String>();
        for (DataSetResult.DataSet ds : result.getData()) {
            List<DataSetResult.DeviceInfo> list = ds.getDeviceInfo().stream().filter(d -> d.getVersionNumber() > 1).collect(Collectors.toList());
            if (list != null && list.size() > 0) {
                for (DataSetResult.DeviceInfo deviceInfo : list)
                    keyList.add(deviceInfo.getDeviceRecordKey());
            }
        }
        if (keyList.size() > 0) {
            DataSetTotalResult res = getHistoryData(keyList);
            if (res.getCode() == 1) {
                if (res.getData().size() > 0) {
                    int count = result.getTotalRecordCount();
                    count = count + res.getTotalRecordCount();
                    result.setTotalRecordCount(count);
                    List<DataSetResult.DataSet> data = result.getData();
                    data.addAll(res.getData());
                    result.setData(data);
                }
            } else {
                JobLog jobLog = new JobLog();
                jobLog.setType("error");
                jobLog.setMsg(res.getMsg());
                jobLog.setCreateTime(new Date());
                jobLogService.insert(jobLog);
            }

        }
        return result;
    }

    //按月份下载 边下载边保存
    public BaseResponse downloadByMonth(String month, int page) {
        try {
            int totalPage = page;
            int totalRecordCount = 0;
            while (page <= totalPage) {
                DataSetResult dataSetResult = getData(month, 2, page);
                if (dataSetResult.getReturnCode() != 1) {
                    String msg = "下载到第" + page + "页发生错误";
                    //发生错误，存储日志
                    JobLog jobLog = new JobLog();
                    jobLog.setDownloadType("manual");
                    jobLog.setType("error");
                    jobLog.setMsg(msg);
                    jobLog.setContent(dataSetResult.getReturnMsg());
                    jobLog.setCreateTime(new Date());
                    jobLogService.insert(jobLog);
                    return ResultVOUtils.error(-1, msg + ",错误信息：" + dataSetResult.getReturnMsg());
                } else if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    //存储数据
                    List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                    dataSets.add(dataSetResult.getDataSet());
                    deviceSaveService.DeviceSave(dataSets, null);
                }
                totalPage = dataSetResult.getTotalPageCount();
                totalRecordCount = dataSetResult.getTotalRecordCount();
                page++;
            }
            JobLog jobLog = new JobLog();
            jobLog.setType("info");
            jobLog.setDownloadType("manual");
            jobLog.setMsg("月份：" + month + ",执行成功！");
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        } catch (Exception e) {
            e.printStackTrace();
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType("manual");
            jobLog.setType("error");
            jobLog.setMsg("月份：" + month + ",执行失败:" + e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return ResultVOUtils.error(-1, jobLog.getMsg());
        }

        return ResultVOUtils.success();
    }


    //按月份下载 边下载边保存
    public BaseResponse downloadByMonthTest(String month, int page) {
        try {
            int totalPage = page;
            int totalRecordCount = 0;
            while (page <= totalPage) {
                DataSetResult dataSetResult = getData(month, 2, page);
                if (dataSetResult.getReturnCode() != 1) {
                    String msg = "下载到第" + page + "页发生错误";
                    //发生错误，存储日志
                    JobLog jobLog = new JobLog();
                    jobLog.setDownloadType("manual");
                    jobLog.setType("error");
                    jobLog.setMsg(msg);
                    jobLog.setContent(dataSetResult.getReturnMsg());
                    jobLog.setCreateTime(new Date());
                    jobLogService.insert(jobLog);
                    return ResultVOUtils.error(-1, msg + ",错误信息：" + dataSetResult.getReturnMsg());
                } else if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    //存储数据
                    List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                    dataSets.add(dataSetResult.getDataSet());
                    for (DataSetResult.DataSet ds : dataSets) {
                        for (DataSetResult.DeviceInfo item : ds.getDeviceInfo()) {
                            Device device = new Device();
                            device.setHchzsb(item.getHchzsb());
                            device.setCplx(item.getCplx());
                            device.setDevicerecordkey(item.getDeviceRecordKey());
                            deviceService.updateCplx(device);


                            ProductInfoEntity productInfoEntity = new ProductInfoEntity();
                            productInfoEntity.setHchzsb(item.getHchzsb());
                            productInfoEntity.setCplx(item.getCplx());
                            productInfoEntity.setDeviceRecordKey(item.getDeviceRecordKey());
                            productInfoService.updateCplx(productInfoEntity);

                        }
                    }
                }
                totalPage = dataSetResult.getTotalPageCount();
                totalRecordCount = dataSetResult.getTotalRecordCount();
                page++;
            }
            JobLog jobLog = new JobLog();
            jobLog.setType("info");
            jobLog.setDownloadType("manual");
            jobLog.setMsg("月份：" + month + ",执行成功！");
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        } catch (Exception e) {
            e.printStackTrace();
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType("manual");
            jobLog.setType("error");
            jobLog.setMsg("月份：" + month + ",执行失败:" + e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return ResultVOUtils.error(-1, jobLog.getMsg());
        }

        return ResultVOUtils.success();
    }

}
