package com.glxp.udidl.admin.service.DataSync;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.*;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.res.udid.DataSetResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleHistoryResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleResult;
import com.glxp.udidl.admin.res.udid.DataSetTotalResult;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class DeviceSyncService {
    @Autowired
    private DeviceSaveService deviceSaveService;
    @Autowired
    private DeviceDownloadService deviceDownloadService;
    @Autowired
    JobLogService jobLogService;                //日志
    @Resource
    ProductInfoService productInfoService;

    public void downloadAndSave(String day, String downloadType) {
        //1.下载唯一标识数据
        DataSetTotalResult result;
        String rangeValue = "";
        if (day.isEmpty())
            rangeValue = DateUtil.getYesterday();
        else
            rangeValue = day;
        result = deviceDownloadService.getData(rangeValue, 1);
        if (result.getCode() != 1) {
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(downloadType);
            jobLog.setDownloadDate(DateUtil.parseDate(rangeValue));
            jobLog.setType("error");
            jobLog.setMsg(result.getMsg());
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return;
        }
        int insertCount = deviceSaveService.DeviceSave(result.getData(), DateUtil.parseDate(rangeValue));
        //2.下载历史记录
        List<String> keyList = new ArrayList<String>();
        for (DataSetResult.DataSet ds : result.getData()) {
            List<DataSetResult.DeviceInfo> list = ds.getDeviceInfo();
            if (list != null && list.size() > 0) {
                for (DataSetResult.DeviceInfo deviceInfo : list)
                    keyList.add(deviceInfo.getDeviceRecordKey());
            }
        }
        if (keyList.size() > 0) {
            DataSetTotalResult res = deviceDownloadService.getHistoryData2(keyList);
            if (res.getCode() == 1) {
                if (res.getData() != null && res.getData().size() > 0) {
                    int count = result.getTotalRecordCount();
                    count = count + res.getTotalRecordCount();
                    result.setTotalRecordCount(count);
                    List<DataSetResult.DataSet> data = result.getData();
                    data.addAll(res.getData());
                    result.setData(data);
                    int hisInsertCount = deviceSaveService.DeviceSave(res.getData(), null);
                    insertCount = insertCount + hisInsertCount;
                }
            } else {
                JobLog jobLog = new JobLog();
                jobLog.setDownloadType(downloadType);
                jobLog.setDownloadDate(DateUtil.parseDate(rangeValue));
                jobLog.setType("error");
                jobLog.setMsg("下载历史记录出错：" + res.getMsg());
                jobLog.setCreateTime(new Date());
                jobLogService.insert(jobLog);
            }

        }

        JobLog jobLog = new JobLog();
        jobLog.setDownloadType(downloadType);
        jobLog.setDownloadDate(DateUtil.parseDate(day));
        jobLog.setTotalCount(result.getTotalRecordCount());
        jobLog.setRealCount(result.realRecordCount());
        jobLog.setInsertCount(insertCount);
        if (result.realRecordCount() != result.getTotalRecordCount()) {
            jobLog.setType("warning");
            jobLog.setMsg("接口总行数和实际行数不一致");
            jobLog.setCreateTime(new Date());
        } else {
            jobLog.setType("info");
            jobLog.setMsg("执行成功！");
            jobLog.setCreateTime(new Date());
        }
        jobLogService.insert(jobLog);
    }

    /**
     * 下载1天数据，并保存
     *
     * @param day
     */
   /* private void  downloadUdi2(String day,String downloadType) {
        DataSetTotalResult result = deviceDownloadService.getData(day,downloadType);
        if(result.getCode() == 1) {
            //存储数据
            int insertCount = deviceSaveService.DeviceSave(result.getData());
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(downloadType);
            jobLog.setDownloadDate(DateUtil.parseDate(day));
            jobLog.setTotalCount(result.getTotalRecordCount());
            jobLog.setRealCount(result.realRecordCount());
            jobLog.setInsertCount(insertCount);
            if(result.realRecordCount() != result.getTotalRecordCount()) {
                jobLog.setType("warning");
                jobLog.setMsg("接口总行数和实际行数不一致");
                jobLog.setCreateTime(new Date());
            } else {
                jobLog.setType("info");
                jobLog.setMsg("执行成功！");
                jobLog.setCreateTime(new Date());
            }
            jobLogService.insert(jobLog);
        }
    }*/
    @Async
    public void downloadUdi(String day, String downloadType) {
        try {
            downloadAndSave(day, downloadType);
            //downloadUdi2(day,downloadType);
        } catch (Exception e) {
            e.printStackTrace();
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(downloadType);
            jobLog.setDownloadDate(DateUtil.parseDate(day));
            jobLog.setType("error");
            jobLog.setMsg(e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        }
    }

    /**
     * 手动下载
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @Async
    public BaseResponse downloadUdi(Date startDate, Date endDate) {
        if (startDate.getTime() > endDate.getTime())
            return ResultVOUtils.error(-1, "起始日期不能大于结束日期!");
        Date fdate = startDate;
        while (fdate.getTime() <= endDate.getTime()) {
            String day = DateUtil.formatDate(fdate);
            downloadUdi(day, "manual");
            fdate = DateUtil.addDays(fdate, 1);
        }
        return ResultVOUtils.success("执行完成，详情请查看日志！");
    }


    public BaseResponse searchDlByDi(String deviceId) {
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setNameCode(deviceId);
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        if (CollUtil.isEmpty(productInfoEntityList)) {
            DataSetSingleResult result1 = deviceDownloadService.downloadByDi(deviceId);
            if (result1.getReturnCode() != 1)
                return ResultVOUtils.error(-1, result1.getReturnMsg());

            if(CollUtil.isNotEmpty(result1.getDataSet().getDeviceInfo()) ){
                List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                dataSets.add(result1.getDataSet());
                deviceSaveService.DeviceSave(dataSets, new Date());
                productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
            }else {
                return ResultVOUtils.error(501,"医疗器械唯一标识数据库未找到此产品DI,请检查是否输入正确或联系厂家是否已上传");
            }

        }
        return ResultVOUtils.success(productInfoEntityList);
    }

    public void dlHistory(DataSetSingleResult result1) {

        DataSetSingleHistoryResult result = new DataSetSingleHistoryResult();
        result.setDataSet(result1.getDataSet());
        if (result1.getDataSet().getDeviceInfo() != null && result1.getDataSet().getDeviceInfo().size() > 0) {
            List<String> keyList = deviceDownloadService.getKeyList(result1.getDataSet());
            if (keyList.size() > 0) {
                DataSetTotalResult result2 = deviceDownloadService.getHistoryData(keyList);
                result.setDataList(result2.getData());
            }
        }
    }


    public BaseResponse downloadByDi(String deviceId) {
        DataSetSingleResult result1 = deviceDownloadService.downloadByDi(deviceId);
        if (result1.getReturnCode() != 1)
            return ResultVOUtils.error(-1, result1.getReturnMsg());
        DataSetSingleHistoryResult result = new DataSetSingleHistoryResult();
        result.setDataSet(result1.getDataSet());
        if (result1.getDataSet().getDeviceInfo() != null && result1.getDataSet().getDeviceInfo().size() > 0) {
            List<String> keyList = deviceDownloadService.getKeyList(result1.getDataSet());
            if (keyList.size() > 0) {
                DataSetTotalResult result2 = deviceDownloadService.getHistoryData(keyList);
                result.setDataList(result2.getData());
            }
        }
        List<DataSetResult.DataSet> dataSets = new ArrayList<>();
        dataSets.add(result1.getDataSet());
        deviceSaveService.DeviceSave(dataSets, new Date());
        return ResultVOUtils.success(result);
    }

    public BaseResponse downloadSingle(String deviceId) {
        DataSetSingleResult res = deviceDownloadService.downloadByDi(deviceId);
        if (res.getReturnCode() != 1)
            return ResultVOUtils.error(-1, res.getReturnMsg());
        List<DataSetResult.DataSet> dataSets = new ArrayList<>();
        dataSets.add(res.getDataSet());
        deviceSaveService.DeviceSave(dataSets, new Date());
        return ResultVOUtils.success(Convert(res.getDataSet()));

    }

    /**
     * 下载单个历史记录，并转换格式
     *
     * @param key
     * @return
     */
    public BaseResponse downloadHistory(String key) {
        List<String> keyList = new ArrayList<>();
        keyList.add(key);
        DataSetTotalResult res = deviceDownloadService.getHistoryData(keyList);
        if (res.getCode() != 1)
            return ResultVOUtils.error(-1, res.getMsg());
        List<DeviceEntity> result = new ArrayList<>();
        for (DataSetResult.DataSet ds : res.getData())
            result.addAll(Convert(ds));
        return ResultVOUtils.success(result);
    }

    /**
     * 格式转换
     *
     * @param dataSet
     * @return
     */
    private List<DeviceEntity> Convert(DataSetResult.DataSet dataSet) {
        List<DeviceEntity> result = new ArrayList<>();
        List<DataSetResult.DeviceInfo> deviceInfoList = dataSet.getDeviceInfo();
        List<DataSetResult.ClinicalInfo> clinicalInfos = dataSet.getClinicalInfo();
        List<DataSetResult.StorageInfo> storageInfos = dataSet.getStorageInfo();
        List<DataSetResult.PackingInfo> packingInfos = dataSet.getPackingInfo();

        if (deviceInfoList != null && deviceInfoList.size() > 0) {
            for (DataSetResult.DeviceInfo ds : deviceInfoList) {
                DeviceEntity deviceEntity = new DeviceEntity();
                BeanUtils.copyProperties(ds, deviceEntity);
                deviceEntity.setDevicerecordkey(ds.getDeviceRecordKey());
                deviceEntity.setVersionnumber(ds.getVersionNumber() + "");
                deviceEntity.setVersionstatus(ds.getVersionStatus());
                deviceEntity.setVersiontime(ds.getVersionTime());
                deviceEntity.setContactlistList(convertList2List(ds.getContactList(), Contactlist.class));
                if (clinicalInfos != null && clinicalInfos.size() > 0) {
                    List<DataSetResult.ClinicalInfo> clinicalInfos1 = clinicalInfos.stream().filter(p -> p.getDeviceRecordKey().equals(ds.getDeviceRecordKey())).
                            collect(Collectors.toList());
                    if (clinicalInfos1 != null && clinicalInfos1.size() > 0) {
                        deviceEntity.setDeviceclinicals(convertList2List(clinicalInfos1, Deviceclinical.class));
                        deviceEntity.getDeviceclinicals().forEach(p -> p.setDevicerecordkey(ds.getDeviceRecordKey()));
                    }

                }
                if (storageInfos != null && storageInfos.size() > 0) {
                    List<DataSetResult.StorageInfo> storageInfos1 = storageInfos.stream().filter(p -> p.getDeviceRecordKey().equals(ds.getDeviceRecordKey())).
                            collect(Collectors.toList());
                    if (storageInfos1 != null && storageInfos1.size() > 0) {
                        deviceEntity.setDevicestorages(convertList2List(storageInfos1, Devicestorage.class));
                        deviceEntity.getDevicestorages().forEach(p -> p.setDevicerecordkey(ds.getDeviceRecordKey()));
                    }

                }
                if (packingInfos != null && packingInfos.size() > 0) {
                    List<DataSetResult.PackingInfo> packingInfos1 = packingInfos.stream().filter(p -> p.getDeviceRecordKey().equals(ds.getDeviceRecordKey())).
                            collect(Collectors.toList());
                    if (packingInfos1 != null && packingInfos1.size() > 0) {
                        deviceEntity.setDevicepackages(convertList2List(packingInfos1, Devicepackage.class));
                        deviceEntity.getDevicepackages().forEach(p -> p.setDevicerecordkey(ds.getDeviceRecordKey()));
                    }

                }
                result.add(deviceEntity);
            }

        }
        return result;
    }

    private <E, T> List<T> convertList2List(List<E> input, Class<T> clzz) {
        List<T> output = new ArrayList<>();
        if (!CollectionUtils.isEmpty(input)) {
            for (E source : input) {
                T target = BeanUtils.instantiate(clzz);
                BeanUtils.copyProperties(source, target);
                output.add(target);
            }
        }
        return output;
    }

    /**
     * 下载从今天开始前几天的数据并保存  自动下载
     *
     * @param days
     */
    public void downloadUdi(int days) {
        for (int i = 1; i <= days; i++) {
            String day = DateUtil.getLastDay(-i);
            downloadUdi(day, "auto");
        }
    }
}
