package com.glxp.udidl.admin.service.DataSync;

import com.alibaba.fastjson.JSON;
import com.glxp.udidl.admin.config.GlobalConfig;
import com.glxp.udidl.admin.entity.info.FileInfoEntity;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.service.info.FileInfoService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@Service
public class ProductInfoDlService {
    @Autowired
    GlobalConfig globalConfig;
    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    FileInfoService fileInfoService;

    public BaseResponse ExportToFile(Date startDate, Date endDate, String downloadType) {
        if (startDate.getTime() > endDate.getTime())
            return ResultVOUtils.error(-1, "起始日期不能大于结束日期！");
        String date0 = DateUtil.formatDate(startDate);
        String date1 = DateUtil.formatDate(DateUtil.addDays(endDate, 1));//结束日期+1天
        List<ProductInfoEntity> list = productInfoService.selectByUpdateTime(date0, date1);
        String DateStr0 = DateUtil.formatDate(startDate, "yyyyMMdd");
        String dateStr1 = DateUtil.formatDate(endDate, "yyyyMMdd");
        //String fileName = "UDI_"+DateStr0+"_"+dateStr1+"_"+list.size()+".json";
        String fileName = "UDI_" + DateStr0 + "_" + dateStr1 + "_" + list.size() + ".zip";
        String path = globalConfig.getDownloadPath() + "/" + fileName;
        BaseResponse result = SaveToZip(list, path);
        if (result.success()) {
            File file = new File(path);
            FileInfoEntity fileInfoEntity = new FileInfoEntity();
            fileInfoEntity.setType("productInfo");
            fileInfoEntity.setFileName(fileName);
            fileInfoEntity.setDownloadType(downloadType);
            fileInfoEntity.setStartDate(startDate);
            fileInfoEntity.setEndDate(endDate);
            fileInfoEntity.setCount(list.size());
            fileInfoEntity.setFileSize(file.length());
            fileInfoEntity.setCreateTime(new Date());
            fileInfoService.insert(fileInfoEntity);
        }
        return result;
       /* try
        {
            if(list.size()>0){
                BufferedWriter bw = new BufferedWriter(new FileWriter(path,false));
                String ss = JSON.toJSONString(list);
                bw.write(ss) ;
                bw.newLine(); //换行用
                bw.close();
                File file = new File(path);
                FileInfoEntity fileInfoEntity = new FileInfoEntity();
                fileInfoEntity.setType("productInfo");
                fileInfoEntity.setFileName(fileName);
                fileInfoEntity.setDownloadType(downloadType);
                fileInfoEntity.setStartDate(startDate);
                fileInfoEntity.setEndDate(endDate);
                fileInfoEntity.setCount(list.size());
                fileInfoEntity.setFileSize(file.length());
                fileInfoEntity.setCreateTime(new Date());
                fileInfoService.insert(fileInfoEntity);
            }else
                return ResultVOUtils.error(-1,"查无数据");

        }catch (Exception e){
            return ResultVOUtils.error(-1,e.getMessage());
        }
       return ResultVOUtils.success();*/
    }

    private BaseResponse SaveToZip(List<ProductInfoEntity> list, String fileName) {
        int count = list.size();
        int limit = 10000;
        int pageTotal = 0;
        if (count % limit == 0) {
            pageTotal = count / limit;
        } else {
            pageTotal = count / limit + 1;
        }
        int page = 1;
        int BUFFER_SIZE = 4096;
        byte[] buffer = new byte[BUFFER_SIZE];
        int readLength;
        try {
            File zipFile = new File(fileName);
            ZipOutputStream zip = new ZipOutputStream(new FileOutputStream(zipFile));
            while (page <= pageTotal) {
            List<ProductInfoEntity> subList = list.stream().skip((page - 1) * limit).limit(limit).collect(Collectors.toList());
            String ss = JSON.toJSONString(subList);
            zip.putNextEntry(new ZipEntry(page + ".json"));
            InputStream input = new ByteArrayInputStream(ss.getBytes(StandardCharsets.UTF_8));
            while ((readLength = input.read(buffer, 0, BUFFER_SIZE)) != -1) {
                zip.write(buffer, 0, readLength);
            }
            zip.flush();
            input.close();
            page++;
            }
            zip.close();
        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(-1, e.getMessage());
        }
        return ResultVOUtils.success();
    }
}
