package com.glxp.udidl.admin.service.sys.impl;

import com.glxp.udidl.admin.dao.sys.SysMenuMapper;
import com.glxp.udidl.admin.dto.sys.MenuTreeModel;
import com.glxp.udidl.admin.dto.sys.SysMenuModel;
import com.glxp.udidl.admin.entity.sys.SysMenu;
import com.glxp.udidl.admin.service.sys.SysMenuService;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SysMenuServiceImpl implements SysMenuService {
    @Autowired
    SysMenuMapper sysMenuMapper;

    @Override
    public BaseResponse getMenuTree() {
        List<SysMenu> menus = sysMenuMapper.selectAll();
        List<MenuTreeModel> resutl = getChild(menus, 0);
        return ResultVOUtils.success(resutl);
    }
    @Override
    public BaseResponse getMenuTreeActive(){
        List<SysMenu> menus = sysMenuMapper.selectActiveAll();
        List<MenuTreeModel> resutl = getChild(menus, 0);
        return ResultVOUtils.success(resutl);
    }
    public BaseResponse insert(SysMenuModel model){
        SysMenu sysMenu = new SysMenu();
        BeanUtils.copyProperties(model,sysMenu);
        sysMenu.setCreateTime(new Date());
        sysMenuMapper.insert(sysMenu);
        return ResultVOUtils.success();
    }
    public BaseResponse update(SysMenuModel model){
        SysMenu sysMenu = sysMenuMapper.selectByPrimaryKey(model.getId());
        if(sysMenu == null)
            return ResultVOUtils.error(-1,"找不到记录");
        BeanUtils.copyProperties(model,sysMenu);
        sysMenuMapper.updateByPrimaryKey(sysMenu);
        return ResultVOUtils.success();
    }
    public BaseResponse detail(Integer id){
        SysMenu sysMenu = sysMenuMapper.selectByPrimaryKey(id);
        if(sysMenu == null)
            return ResultVOUtils.error(-1,"找不到记录");
        return ResultVOUtils.success(sysMenu);
    }
    public BaseResponse delete(Integer id){
        if(sysMenuMapper.getChildCount(id)>0)
            return ResultVOUtils.error(-1,"存在子级，请先删除子级！");
        sysMenuMapper.deleteByPrimaryKey(id);
        return ResultVOUtils.success();
    }
    private List<MenuTreeModel> getChild(List<SysMenu> menus, Integer pId) {
        List<MenuTreeModel> result = new ArrayList<>();
        List<SysMenu> list = menus.stream().filter(p -> p.getParentId().equals(pId)).collect(Collectors.toList());
        if (list != null && list.size() > 0) {
            for (SysMenu item : list) {
                MenuTreeModel treeModel = new MenuTreeModel();
                BeanUtils.copyProperties(item, treeModel);
                List<MenuTreeModel> child = getChild(menus, treeModel.getId());
                if (child != null && child.size() > 0)
                    treeModel.setChildren(child);
                result.add(treeModel);
            }
        }
        return result;
    }
}
