package com.glxp.udidl.admin.service.sys.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.sys.SysUserMapper;
import com.glxp.udidl.admin.dto.sys.SysUserModel;
import com.glxp.udidl.admin.dto.sys.SysUserParam;
import com.glxp.udidl.admin.entity.sys.SysUser;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.sys.SysUserService;
import com.glxp.udidl.common.res.BaseResponse;
import com.glxp.udidl.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service
public class SysUserServiceImpl implements SysUserService {
    @Autowired
    SysUserMapper mapper;
    @Override
    public BaseResponse list(SysUserParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        List<SysUser> list = mapper.list(param);
        PageInfo<SysUser> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<SysUser> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Override
    public BaseResponse insert(SysUserModel model) {
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(model, sysUser);
        sysUser.setCreateTime(new Date());
        sysUser.setUserKey(getUUId());
        mapper.insert(sysUser);
        return ResultVOUtils.success();
    }

    @Override
    public BaseResponse update(SysUserModel model) {
        SysUser sysUser = mapper.selectByPrimaryKey(model.getId());
        if (sysUser == null)
            return ResultVOUtils.error(-1, "找不到记录");
        BeanUtils.copyProperties(model, sysUser);
        mapper.updateByPrimaryKey(sysUser);
        return ResultVOUtils.success();
    }

    @Override
    public BaseResponse detail(Integer id) {
        SysUser sysUser = mapper.selectByPrimaryKey(id);
        if (sysUser == null)
            return ResultVOUtils.error(-1, "找不到记录");
        return ResultVOUtils.success(sysUser);
    }

    @Override
    public BaseResponse delete(Integer id) {
        mapper.deleteByPrimaryKey(id);
        return ResultVOUtils.success();
    }
    private String getUUId() {
        UUID uuid = UUID.randomUUID();
        return uuid.toString().replace("-", "");
    }
}
