package com.glxp.udidl.admin.thread;

import com.glxp.udidl.admin.dao.info.ScheduledDao;
import com.glxp.udidl.admin.entity.info.ScheduledEntity;
import com.glxp.udidl.admin.entity.udid.JobLog;
import com.glxp.udidl.admin.req.info.ScheduledRequest;
import com.glxp.udidl.admin.service.DataSync.ProductInfoDlService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.common.res.BaseResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 保存json数据到文件
 */
@Component
@EnableScheduling
public class DownloadProductInfoTask implements SchedulingConfigurer {
    final Logger logger = LoggerFactory.getLogger(DownloadProductInfoTask.class);
    /* @Autowired
     GlobalConfig globalConfig;
     @Autowired
     ProductInfoService productInfoService;
     @Autowired
     FileInfoService fileInfoService;*/
    @Autowired
    ProductInfoDlService productInfoDlService;
    @Resource
    private ScheduledDao scheduledDao;
    @Autowired
    JobLogService jobLogService;
    @Resource
    UdiCompanyTask udiCompanyTask;

    //日志
   /* public void SaveToFile()  throws IOException {
        String updateTime = DateUtil.getLastDay(-7);
        String fromDate = DateUtil.getDate("yyyyMMdd",-7,5);
        String endDate = DateUtil.getDate("yyyyMMDD");
        List<ProductInfoEntity> list = productInfoService.selectByUpdateTime(updateTime);
        String fileName = "UDI_"+fromDate+"_"+endDate+"_"+list.size()+".json";
        String path = globalConfig.getDownloadPath()+"/"+fileName;
        BufferedWriter bw = new BufferedWriter(new FileWriter(path,false));
        String ss = JSON.toJSONString(list);
        bw.write(ss) ;
        bw.newLine(); //换行用
        bw.close();
        File file = new File(path);
        FileInfoEntity fileInfoEntity = new FileInfoEntity();
        fileInfoEntity.setType("productInfo");
        fileInfoEntity.setFileName(fileName);
        fileInfoEntity.setCount(list.size());
        fileInfoEntity.setFileSize(file.length());
        fileInfoEntity.setCreateTime(new Date());
        fileInfoService.insert(fileInfoEntity);
    }*/


    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("downloadProductInfo");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    String cron = scheduledEntity.getCron();
                    if (cron.isEmpty()) {
                        logger.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }

    private void process() {
        Date endDate = DateUtil.parseDate((DateUtil.formatDate(new Date())));
        Date startDate = DateUtil.addDays(endDate, -7);
        BaseResponse res = productInfoDlService.ExportToFile(startDate, endDate, "auto");
        if (res.getCode() != 20000) {
            JobLog jobLog = new JobLog();
            jobLog.setType("error");
            jobLog.setDownloadType("auto");
            jobLog.setMsg("下载文件失败:" + res.getMessage());
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        }

        //更新一下企业信息
        udiCompanyTask.transAll();

        /*try{
            SaveToFile();
        }catch (Exception e){
            e.printStackTrace();
            JobLog jobLog = new JobLog();
            jobLog.setType("error");
            jobLog.setDownloadType("auto");
            jobLog.setMsg("下载文件失败:"e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        }*/
    }
}
